import unittest
from onetimepad import onetimepad
from cryptography.fernet import Fernet


class TestOnetimepad(unittest.TestCase):

    def test_string_lengths_must_match(self):
        with self.assertRaises(AssertionError):
            onetimepad('abc', 'defg')

    def test_keys_are_retrievable(self):
        k1 = Fernet.generate_key()
        k2 = Fernet.generate_key()
        encrypted = onetimepad(k1, k2)
        self.assertEqual(k1, onetimepad(encrypted, k2))

    def test_symmetry_of_onetime_pad(self):
        k1 = Fernet.generate_key()
        k2 = Fernet.generate_key()
        encrypted = onetimepad(k1, k2)
        self.assertEqual(k1, onetimepad(encrypted, k2))
        self.assertEqual(k1, onetimepad(k2, encrypted))
        self.assertEqual(k2, onetimepad(k1, encrypted))
        self.assertEqual(k2, onetimepad(encrypted, k1))



"""
A library for securely saving and loading images to S3
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
setup(
    name='otpad',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version='0.1.1',

    description='A small util for doing one-time pad',

    # The project's main homepage.
    url='https://github.com/erikreppel/otp',
    download_url='https://github.com/erikreppel/otpad/archive/0.1.1.tar.gz',

    # Author details
    author='Erik Reppel',
    author_email='erikreppel@gmail.com',

    # Choose your license
    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 5 - Production/Stable',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2.7',
    ],

    # What does your project relate to?
    keywords='encryption',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),

    # Alternatively, if you want to distribute just a my_module.py, uncomment
    # this:
    #   py_modules=["my_module"],

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=['cryptography'],

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    extras_require={
        'test': ['nose'],
    }
)



from operator import xor


def onetimepad(key1, key2):
    '''
    Performs a one time pad on two strings.
    :params - key1: key 1 and key 2 are strings of equal length
            - key2: key 1 and key 2 are strings of equal length
    :returns - the resultant from the XOR as a string
    Note: key1 and key2 must be the same length
          and that operations are symmetric
    '''
    assert len(key1) == len(key2)
    key1_bytes, key2_bytes = bytearray(key1), bytearray(key2)
    padded = bytearray([1] * len(key1_bytes))

    for i, _ in enumerate(key1_bytes):
        padded[i] = xor(key1_bytes[i], key2_bytes[i])
    padded = padded.decode('utf-8')
    return str(padded)
